/* Program for generating test data for ANN programs.
+ 
+  Michael Neidhardt, Oct. 2008. */
#include <iostream>
#include <vector>
#include "Point.h"
#include "Datasource2.h"
#include "annutil.h"

using namespace std;
typedef double T;

int main(int argc, char** argv) {

   if (argc < 4) {
      cerr << "\nCreates n points of type t with precision p and writes them to filename.\n"
           << "Precision is only applicable for t=5.\n"
           << "Syntax: " << argv[0] << " t n filename [p]\n\n"
           << "   t=1 creates n uniform random double points in [-2^53, 2^53-1].\n"
           << "   t=2 creates n random double points on unit square.\n"
           << "   t=3 creates n random double points in unit circle.\n"
           << "   t=4 creates n random double points approx. on the unit circle.\n\n"
           << "   t=5 creates n uniform random integer points with precision p in [0,2^31-1].\n"
           << "       If p is not present, it defaults to 31. Mainly for planesweep algorithms.\n"
           << "   t=6 creates n random integer points (stored in doubles) in ]-2^51, 2^51[.\n"
           << "       Mainly for TTL/ABDPY which handles integers in this range (stored in doubles).\n"
           << "   t=7 creates n random integer points in a circle w. radius 1000.\n"
           << "   t=8 creates n random integer points approx. on the circle w. radius 1000.\n"
           << "   t=9 creates n random double points approx. on a line.\n\n";

      exit(0);
   }

   Datasource2<T,Point<T> > dsrc;
   vector<Point<T> > pointset;
   int type = atoi(argv[1]);
   int n = atoi(argv[2]);
   char* outfile = argv[3];
   int prec=31;
   
   switch (type) {
   case 1:
      pointset = dsrc.getRandomDoublePoints(n, false);
      break;
   case 2:
      pointset = dsrc.getRandomDoublePoints(n, true);
      break;
   case 3:
      pointset = dsrc.getRDInUnitCircle(n);
      break;
   case 4:
      pointset = dsrc.getRDOnUnitCircle(n);
      break;
   case 5:
      if (argc == 5) { prec = atoi(argv[4]); }
      pointset = dsrc.getRandomIntPoints(n, prec);
      break;
   case 6:
      pointset = dsrc.getRandomABDPYPoints(n);
      break;
   case 7:
      pointset = dsrc.getRIInUnitCircle(n);
      break;
   case 8:
      pointset = dsrc.getRIOnUnitCircle(n);
      break;
   case 9:
      pointset = dsrc.getRDOnLine(n);
      break;
   default:
      std::cout << "Something is wrong. Run with no args to see syntax.\n";
      exit(0);
   }

   dsrc.writePointset(pointset, outfile);

   return 0;
}

