#!/usr/bin/perl -w

#----------------------------------------------------------
# The idea is to be able to run a program for a series of different input sizes, say
# 1mill, 2mill,...,10mill points. To get more certain timings, each run needs to be 
# repeated x times, so for each input size I run the program 10 times. The time used is then
# the average times used over the 10 runs.
#-------------------------------------------------------------------------------------------

# $runs is the number of times to run the individual program for a given input file. 
# I run the program several times for each input to get a more reliable average time.
my $runs=4;
my $progname;
my $file;
my $idstring;
my $datafile;
my $buildtime;

if ($#ARGV != 2) { 
   print "\nSyntax: $0 programName wildcard ID-string.\n\nwildcard is for the files you want to run, and\n",
         "ID-string is a text that identifies the program. Will be used as filename for the gnuplot-files\n" .
         "and printed in the output.\n"; 
   exit(0);
}
else {
   ($progname, $wildcard, $idstring) = @ARGV;
   $progname = "/home/michael/doks/c++/ann/" . $progname;
   @filenames = glob($wildcard);          # The files to read points from.
   
   if ($idstring =~ m/^[a-z0-9]+$/) { ; }    # print "OK: ",$idstring,"\n";}
   else {
      print "ID-string can only contain letters a-z.\n";
      exit(1);
   }
   
   $datafile = $idstring . ".dat";
   open(OUT, ">$datafile");

   foreach $file (@filenames) {
      $buildtime = runProg($file);
   }

   close(OUT);
   
   doPlot($buildtime);
}

#----------- SUBS Herunder--------------------

sub runProg {
   my $file = shift;
   $buildtime=0.0; $minbuild=999999999.0; $maxbuild=0.0;
   $anntime=0.0;   $minann=999999999.0;   $maxann=0.0;
   $totaltime=0.0; $mintot=999999999.0;   $maxtot=0.0;

   print "Program: $progname Inputfile=$file.\n";

   for ($i=0; $i<$runs; $i++) {
      $res = `$progname 0 $file 0 1 0`;
      if ($res =~ /n=(\d+)\s+Rsz=(\d+)\s+Prec=(\d+)\s+Buildtime=(\d+\.\d+)\s+ANNTime=(\d+\.\d+)\s+Totaltime=(\d+\.\d+)/i) {
         $n          = $1; # Is the same in every run on the same input.
         #$resultsize = $2; # Not used here.
         #$precision  = $3; # Not used here.
         $buildtime += $4;
         $anntime   += $5;
         $totaltime += $6;
         if ($4 > $maxbuild) { $maxbuild = $4; }
         if ($4 < $minbuild) { $minbuild = $4; }
         if ($5 > $maxann)   { $maxann = $5; }
         if ($5 < $minann)   { $minann = $5; }
         if ($6 > $maxtot)   { $maxtot = $6; }
         if ($6 < $mintot)   { $mintot = $6; }
      }
      print $res;    # Show full output of program.
      # print (($i+1)," ");
   }
   print "\n";

   $buildavg = $buildtime/$runs;
   $annavg   = $anntime/$runs;
   $totalavg = $totaltime/$runs;
   print OUT "$n $buildavg $annavg $totalavg ",($mintot/$totalavg)," ",($maxtot/$totalavg),"\n";
   
   return $buildtime;      # If $buildtime is zero, I dont want a curve for it. That is why I return
                           # the value from here, so doPlot can test it.
}

sub doPlot {
   my $buildtime = shift;
   # print "Plot says: $buildtime\n";
   
   my $outputfile = $idstring . ".ps";

   open(OUT2, ">$idstring".".gp");  # Run this by gnuplot to create a postscript file.
   print OUT2 
         "set xtic auto\n",
         "set ytic auto\n",
         "set title 'Timing for $idstring.'\n",
         "set xlabel 'Punkter'\n",
         "set ylabel 'Sekunder'\n",
         "set key top left inside\n";

   if ($buildtime > 0) {         # This means the algorithm has a build phase and 
      print OUT2                 # a ANN phase, such as triangulation.
         "plot '$datafile' using 1:2 title 'Konstruktion' with linespoints, ",
         "'$datafile' using 1:3 title 'ANN' with linespoints, ",
         "'$datafile' using 1:4 title 'Total' with linespoints\n";
   }
   else {                        # This means the algorithm does everything in one phase,
      print OUT2                 # such as plane sweep.
         "plot '$datafile' using 1:4 title 'ANN' with linespoints\n";
   }

   print OUT2
         "set terminal postscript\n",
         "set output '$outputfile'\n",
         "replot\n";

   close(OUT2);
   
   system("/usr/bin/gnuplot ./$idstring" . ".gp");
}
