/* Class containing utilities for plane sweep, including
+  a function for printing out bisectors in a binary tree
+  to postscript.
*/

//#include "Point.h"
//#include "Bisector.h"

template< typename T >
class PSutil {
private:
   typedef typename map<Bisector<T>, int, less<Bisector<T> > >::iterator mmap_iter;
   typedef map<Bisector<T>, int, less<Bisector<T> > > bbstree;
   int minY, maxY;
   
public:
   PSutil() {
      minY=-1000;
      maxY=1000;
   }

   void doPostscript(const Bisector<T>& bs, Point<T>& p, bool printcoords);
   void inOrder(bbstree& ytable, Point<T>& p);
   void inOrder(bbstree& ytable, T x);
   bool errorInYtable(bbstree& ytable, T x);
};



template<typename T>// --- Prints out a bisector in postscript.
void PSutil<T>::doPostscript(const Bisector<T>& bs, Point<T>& p, bool printcoords=false) {

   T ux = bs.getUpperpoint().x();
   T uy = bs.getUpperpoint().y();
   T lx = bs.getLowerpoint().x();
   T ly = bs.getLowerpoint().y();

   //double x1 = p.x() - 10;
   double x1;
   // Find x-midpoint between two points of the bisector.
   // Thats where I start drawing the bisector.
   if (ux > lx) x1 = lx+(static_cast<double>(ux-lx))/2.0;
   else         x1 = ux+(static_cast<double>(lx-ux))/2.0;
   double x2 = p.x() + 10;
   double y1, y2;

   if (bs.get_b() == 0) { 
      x1 = lx - std::abs(lx-ux)/2.0;
      x2 = x1;
      y1 = uy-5;
      y2 = uy+5;
   }
   else {
      y1 = bs.getYValueAsDouble(x1);
      y2 = bs.getYValueAsDouble(x2);
   }
   
   std::cout << "newpath " << x1 << " " << y1 << " moveto\n";
   std::cout << x2 << " " << y2 << " lineto stroke\n";

   // Print upper point and possibly its coordinates:
   std::cout << "newpath\n";
   std::cout << ux << " " << uy << " moveto "
             << ux << " " << uy << " " << " 0.5 0 360 arc stroke\n";

   if (printcoords) {
      int xposForUPCoords = -170;   // (ux-5);
      std::cout << xposForUPCoords << " " << (uy-0) << " moveto (" << ux << "," << uy << ") show\n"; 
   }
                   
   // Print lower point and possibly its coordinates:
   std::cout << "newpath\n";
   std::cout << lx << " " << ly << " moveto "
                << lx << " " << ly << " " << " 0.5 0 360 arc stroke\n";
   if (printcoords) {
      int xposForLPCoords = -170;   // (lx-5);
      std::cout << xposForLPCoords << " " << (ly-0) << " moveto (" << lx << "," << ly << ") show\n";
   }   
}

// --- Prints out ytable in order and produces postscript of the content.
template<typename T>
void PSutil<T>::inOrder(bbstree& ytable, Point<T>& p) {
   std::cout << "%\n%\n%-------------In-order traversal & POSTSCRIPT dump--------------\n";

   std::cout << "%!PS-Adobe-2.0 EPSF-2.0\n"
        << "% Size of ytable = " << ytable.size() << "\n"
        << "2.9 2.9 scale	% scale coordinate system(?)\n"
        << "250 15 translate	% put origin here\n"
        << "% part 2: draw a filled path\n"
        << "0 0 0.5 setrgbcolor\n"
        << "%----------------\n"
        << "0.1 setlinewidth\n"
        << "newpath 0 -5 moveto 0 " << maxY << " lineto stroke\n"  // Y axis
        << "newpath " << (-300) << " 0 moveto " << 300 << " 0 lineto stroke\n"  // X axis
        << "newpath " << p.x() << " 0 moveto " << p.x() << " " << maxY << " lineto stroke\n"  // Sweep line
        << "% part 3: now we'll put up some text\n"
        << "/Helvetica findfont 2 scalefont setfont\n"
        << "-6 -15 moveto (0,0) show % Mark origo\n"
        << p.x() << " -8 moveto (L=" << p.x() << ") show\n"          // Show sweep line.
        << "0.2 setlinewidth\n";

   // This prints out the axes with marks for every 5 units:

   int marklength=2;     // The length of the axis marks.
   int xposForYAxis=-180;

   cout << "newpath -500 0 moveto 500 0 lineto stroke\n"
        << "newpath " << xposForYAxis << " 0 moveto " << xposForYAxis << " 500 lineto stroke\n"
        << "0.1 setlinewidth\n";

   for (int y=5; y<305; y+=5) {
      cout << "newpath " << (xposForYAxis-marklength) << " " << y 
           << " moveto " << (xposForYAxis+marklength) << " " << y << " lineto stroke\n"
           << xposForYAxis << " " << (y-2) << " moveto (" << y << ") show\n";
   }
   for (int x=-500; x<500; x+=5) {
      cout << "newpath " << x << " " << (-marklength) 
           << " moveto " << x << " " << marklength << " lineto stroke\n";
      if (x%10 == 0) cout << x << " -6 moveto (" << x << ") show\n";
   }
   // End coordinate system.

   std::cout << "% p=[" << p.x() << "," << p.y() << "\n";

   for (mmap_iter iter = ytable.begin(); iter != ytable.end(); iter++) {
      iter->first.print("% BS= ", p.x());
      doPostscript(iter->first, p, true);
   }


   std::cout << "%------------------------------\n"
        << "showpage	% Marks end of page\n";

   std::cout << "%----------End Of in-order-traversal----------\n\n"; 
}

template<typename T>// --- Prints out ytable in order:
void PSutil<T>::inOrder(bbstree& ytable, T x) {
   std::cout << "%\n%-------------In-order traversal--------------\n";
   T minval = -200000000;

   for (mmap_iter iter = ytable.begin(); iter != ytable.end(); iter++) {
      iter->first.print("% ", x);
      
      if (iter->first.get_b() != 0) {
         T yval = iter->first.getYValueAsDouble(x);
         
         if (yval < minval && yval >= minY && yval <= maxY) {
            std::cout << "% ERROR IN YTABLE\n";
            //break;
         }
      }
      if (iter->first.get_b() != 0) 
         minval = iter->first.getYValueAsDouble(x);
   }
   std::cout << "%----------End Of in-order-traversal----------\n\n"; 
}

template<typename T>// --- Checks if ytable is in right order.
bool PSutil<T>::errorInYtable(bbstree& ytable, T x) {
   bool first=true;
   bool err=false, inf=false;
   double yval=0.0;

   for (mmap_iter iter = ytable.begin(); iter != ytable.end(); iter++) {
      if (first) {
         first=false;
         if (iter->first.get_b() == 0) { inf=true; }
         else { yval = iter->first.getYValueAsDouble(x); }
      }
      else {
         if (inf) {
            if (iter->first.get_b() != 0) return true;
         }
         else {
            double tmp;
            if (iter->first.get_b() != 0) {
               tmp = iter->first.getYValueAsDouble(x);
               if (tmp < yval) { return true; }
               yval = tmp;
            }
            else {
               inf = true;
            }
         }
      }
   }
   
   return err;
}

