/* Class that is used for storing in priority queue.
+  Originally made for the ANN Plane sweep algorithm.
+  This is what I store in the prio-queue, and the 2 functions, 
+  pointless and pointmore, are used as compare functions.
+  When the point in question is a data point, index >= 0 and is index
+  to the point in its original vector (possibly unnecessary).
+  When the point is an intersection between two bisectors, index < 0.
+  The two bisectors are represented by upperparent and lowerparent, respectively.
+
+  Michael Neidhardt June 2008.
*/

#ifndef QUEUEOBJECT_H
#define QUEUEOBJECT_H

#include <iostream>
#include <string>

template< typename T >
class QueueObject {
   public:
   QueueObject();
   QueueObject(Point<T> p, int i);
   QueueObject(Point<T> p, int i, Bisector<T> up, Bisector<T> lp);

   void print() const { print(""); }
   void print(string info) const;

   Point<T> point;
   int index;
   Bisector<T> upperparent;
   Bisector<T> lowerparent;
};

template< typename T >
bool operator == (const QueueObject<T>& q1, const QueueObject<T>& q2);
template< typename T >
bool operator != (const QueueObject<T>& q1, const QueueObject<T>& q2);
template< typename T >
bool operator<(const QueueObject<T>& q1, const QueueObject<T>& q2); 
template< typename T >
bool operator>(const QueueObject<T>& q1, const QueueObject<T>& q2);

template< typename T >
QueueObject<T>::QueueObject() {
}

template< typename T >
QueueObject<T>::QueueObject(Point<T> p, int i) {
   point = p;
   index = i;
}

template< typename T >
QueueObject<T>::QueueObject(Point<T> p, int i, Bisector<T> up, Bisector<T> lp) {
   point = p;
   index = i;
   upperparent = up;
   lowerparent = lp;
}

template< typename T >
void QueueObject<T>::print(string info) const {
   std::cout << info << " ";
   point.print();
   std::cout << " " << index << "\n";
}

/* ---------- Non member stuff: ---------- */
template<typename T>
bool operator==(const QueueObject<T>& q1, const QueueObject<T>& q2) {
   if (q1.index >= 0 && q2.index >= 0) {
      return q1.point == q2.point;
   }
   else if (q1.index < 0 && q2.index < 0) {
      return q1.point == q2.point &&
             q1.upperparent == q2.upperparent &&
             q1.lowerparent == q2.lowerparent;
   }
   else
      return false;
}

template<typename T>
bool operator!=(const QueueObject<T>& q1, const QueueObject<T>& q2) {
   if (q1.index >= 0 && q2.index >= 0) {
      return q1.point != q2.point;
   }
   else if (q1.index < 0 && q2.index < 0) {
      return q1.point != q2.point ||
             q1.upperparent != q2.upperparent ||
             q1.lowerparent != q2.lowerparent;
   }
   else
      return true;
}

template< typename T > // For min-prio-queue.
bool operator<(const QueueObject<T>& q1, const QueueObject<T>& q2) { 
   if (q1.point.x() != q2.point.x()) {
      return q1.point.x() > q2.point.x();     // x sorted ascending.
   }
   else if (q1.point.y() != q2.point.y()) {
      return q1.point.y() > q2.point.y();     // y sorted ascending.
   }
   else {                                 // intersection is less than data point.
      return (q1.index < 0 && q2.index >= 0);
   }
}



 // For min-prio-queue with points mirrored.
// The only difference from "<" above is that y-coords are sorted descending here.
template< typename T > 
bool operator>(const QueueObject<T>& q1, const QueueObject<T>& q2) { 
   if (q1.point.x() != q2.point.x()) {
      return q1.point.x() > q2.point.x();     // x sorted ascending.
   }
   else if (q1.point.y() != q2.point.y()) {   // y sorted descending.
      return q1.point.y() < q2.point.y();     // This is the opposite of operator "<" above
   }                                      // to catch points with same y-coord.
   else {                                 // Intersection is less than data point.
      return (q1.index < 0 && q2.index >= 0);
   }
}

#endif
