/* This is a utility containing various helperfunctions for ANN-programs.
+
+  For the print-functions, he point is to get a unified look of the output,
+  so it's easier to make automated statistics on the output.
*/


#ifndef ANNREPORTUTIL_H
#define ANNREPORTUTIL_H

#include <iostream>
//#include <stdlib.h>
#include <sys/time.h>


using namespace std;



namespace annutil {

   /* Get time in microseconds. */
   double microsec() {
     struct timeval t;
     gettimeofday( &t,0 );
     return t.tv_sec * 1000000.0 + t.tv_usec;
   }

   /* Get time in seconds. */
   double microsec(double timethen) {
     return (microsec()-timethen)/1000000.0;
   }


   /* These thwo print functions are there to make the output of several programs
   +  for ANN look the same, and to make it easy to do stats on the results.
   +  In the following, Rsz is the size of the result set. For some algorithms, such
   +  as triangulations, duplicate points are discarded, and so when triangulating a 
   +  pointset with n points, the result of the ANN-algo might contain fewer than n points. */
   void printResultWithCheck(std::string progname, int numOfPoints, int resultsize, 
                             int precision, int errors,
                             double buildtime, double anntime, double checktime) {

      std::cout.setf(ios::fixed);

      std::cout << progname 
           << " n=" << numOfPoints 
           << " Rsz=" << resultsize
           << " Prec=" << precision
           << " Errors=" << errors
           << " Buildtime=" << buildtime 
           << " ANNTime=" << anntime 
           << " Checktime=" << checktime
           << " Totaltime="  << (buildtime+anntime+checktime)
           << ".\n";
   }

   void printResultWithoutCheck(std::string progname, int numOfPoints, int resultsize, 
                                int precision, double buildtime, double anntime) {

      std::cout.setf(ios::fixed);

      std::cout << progname 
           << " n="       << numOfPoints
           << " Rsz="   << resultsize 
           << " Prec="         << precision 
           << " Buildtime="    << buildtime 
           << " ANNTime="      << anntime 
           << " Totaltime="        << (buildtime+anntime) 
           << ".\n";
   }

}; // End of annutil namespace scope

#endif
