/* Class containing utilities for numerical stuff
+  such as showing the binary representation of a float/double.
*/

#ifndef BINUTIL_H
#define BINUTIL_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>

class binutil {
public:
   binutil() {}
   void showBinaryRep(double number);
   void doubleprint(double number);
   void doubleprint2(double number);
   int extractExponent(double number);
   double microsec();
   double microsec(double timethen);
};


void binutil::showBinaryRep(double number) {
    // Copy bits to byte array - written by Jacob Berggreen Jensen.
    unsigned char bytes[sizeof(number)];
    memcpy(bytes, &number, sizeof(number));
    
    int i,j;
    unsigned int mask = 128;
    
    // The 8 most significant bits from d is in bytes[7]
    // The 8 least significant bits from d is in bytes[0]
    // Print binary representation to user
    for(i=7; i>=0; i--) {
        unsigned int byte = (unsigned int)bytes[i];

        for(j=7; j>=0; j--) {
            if((byte & mask) > 0) {
                printf("1");
            }
            else {
                printf("0");
            }
            
            byte = byte << 1;
        }
    }
    printf("\n");
}


void binutil::doubleprint(double number) {
  /* Written by J.R. Shewchuk. */

  unsigned long long no;
  unsigned long long sign, expo;
  int exponent;
  int i, bottomi;

  no = *(unsigned long long *) &number;
  sign = no & 0x8000000000000000ll;
  expo = (no >> 52) & 0x7ffll;
  exponent = (int) expo;
  exponent = exponent - 1023;
  if (sign) {
    printf("-");
  } else {
    printf(" ");
  }
  if (exponent == -1023) {
    printf(
      "0.0000000000000000000000000000000000000000000000000000_     (   )");
  } else {
    printf("1.");
    bottomi = -1;
    for (i = 0; i < 52; i++) {
      if (no & 0x0008000000000000ll) {
        printf("1");
        bottomi = i;
      } else {
        printf("0");
      }
      no <<= 1;
    }
    printf("_%d  (%d)", exponent, exponent - 1 - bottomi);
  }
}

/* Slightly modified doubleprint - this version prints out the bits even when
+  the exponent equals -1023, ie. when its a subnormal. */
void binutil::doubleprint2(double number) {
  unsigned long long no;
  unsigned long long sign, expo;
  int exponent;
  int i, bottomi;

  no = *(unsigned long long *) &number;
  sign = no & 0x8000000000000000ll;
  expo = (no >> 52) & 0x7ffll;
  exponent = (int) expo;
  exponent = exponent - 1023;
  if (sign) { printf("-"); }
  else      { printf(" "); }

  if (exponent == -1023) { printf("0."); }
  else                   { printf("1."); }

  bottomi = -1;
  for (i = 0; i < 52; i++) {
    if (no & 0x0008000000000000ll) {
      printf("1");
      bottomi = i;
    }
    else {
      printf("0");
    }
    no <<= 1;
  }
  printf("_%d  (%d)", exponent, exponent - 1 - bottomi);
}

/* Extracts the exponent of a double. */
int binutil::extractExponent(double number) {
   unsigned long long no;
   unsigned long long expo;
   int exponent;

   no = *(unsigned long long *) &number;
   expo = (no >> 52) & 0x7ffll;
   exponent = (int) expo;

   return exponent-1023;
}


/* Get time in microseconds. */
double binutil::microsec() {
  struct timeval t;
  gettimeofday( &t,0 );
  return t.tv_sec * 1000000.0 + t.tv_usec;
}

/* Get time in microseconds. */
double binutil::microsec(double timethen) {
  struct timeval t;
  gettimeofday( &t,0 );
  double timenow = t.tv_sec * 1000000.0 + t.tv_usec;
  return (timenow-timethen)/1000000.0;
}



#endif
