/* Wrapper class for std::priority_queue.
+  Originally made for the ANN plane sweep algorithm.
+
+  Mostly made by L.O. Belhage.
+
*/

#ifndef PRIORITYQUEUEWRAPPER_H
#define PRIORITYQUEUEWRAPPER_H

   template< typename T >
   class PQ {
      public:
      PQ() {}
      virtual void push(QueueObject<T>& qobj) = 0;
      virtual QueueObject<T> top()            = 0;
      virtual void pop()                      = 0;
      virtual bool empty()                    = 0;
      virtual unsigned int size()             = 0;
      virtual ~PQ() {};
   };
   /*------------------------------------*/

   template< typename T >
   class minPQ : public PQ<T> {
      public:
      minPQ()                         {}
      void push(QueueObject<T>& qobj) { queue.push(qobj); }
      QueueObject<T> top()            { return queue.top(); }
      void pop()                      { queue.pop(); }
      bool empty()                    { return queue.empty(); }
      unsigned int size()             { return queue.size(); }
      ~minPQ()                        {}

      private:
      std::priority_queue<QueueObject<T>, vector<QueueObject<T> >, less<QueueObject<T> > > queue;
   };
   /* ----------------- */

   template< typename T >
   class maxPQ : public PQ<T> {
      public:
      maxPQ()                         {}
      void push(QueueObject<T>& qobj) { queue.push(qobj); }
      QueueObject<T> top()            { return queue.top(); }
      void pop()                      { queue.pop(); }
      bool empty()                    { return queue.empty(); }
      unsigned int size()             { return queue.size(); }
      ~maxPQ()                        {}

      private:
      std::priority_queue<QueueObject<T>, vector<QueueObject<T> >, greater<QueueObject<T> > > queue;
   };
   
#endif
