/* Program der finder ANN med brug af kdtræer.
+  Michael Neidhardt, Oct. 2008. */
#include <iostream>
#include <vector>
#include "orderstats.h"
#include "Point.h"
#include "Datasource2.h"
#include "ANNComparator2.h"
#include "annutil.h"
#include "binutil.h"
#include "kdtree.h"

using namespace std;
typedef double T;

int main(int argc, char** argv) {
   std::string progname = "KDtree";

   if (argc < 2) {
      cerr << progname
           << "\nSyntax: " << argv[0] << " pointtype [filename|numOfPoints]"
           <<                            " [precision] [verbose] [check]\n\n"
           << "   pointtype=0 means the following arg is filename with points.\n" 
           << "   pointtype=998 means use KMPSY-points with 2 big numbered points.\n" 
           << "   pointtype=999 means use KMPSY-points without big numbered points.\n" 
           << "   precision: N/A here. (Precision of random int numbers).\n"
           << "   verbose: 0=quiet, 1=one-line summary, 2=all points listed.\n"
           << "   check:   1 means check against brute force results, 0 means no check.\n\n";

      exit(0);
   }

   Datasource2<T,Point<T> > dsrc;
   vector<Point<T> > pointset;
   
   if (atoi(argv[1]) == 0) {
      pointset = dsrc.readPointset(argv[2]);
   }
   else if (atoi(argv[1]) == 998) {
      pointset = dsrc.getKMPSY1(true);
   }
   else if (atoi(argv[1]) == 999) {
      pointset = dsrc.getKMPSY1(false);
   }
   else {      // Now the data must be made and stored in a file. Then this program reads the file.
      std::cerr << "Input must come from a file. First arg. should be 0.\n";
      exit(1);
   }

   KDTree<T, Point<T>, OrderStats<T,Point<T> > > mytree;
   double buildtime = annutil::microsec();
   mytree.buildTree(pointset);
   buildtime = annutil::microsec(buildtime);
   //mytree.inOrderWalk();

   double anntime = annutil::microsec();
   vector<pair<Point<T>,Point<T> > > annresult;

   for (unsigned int j=0; j<pointset.size(); j++) {
      Point<T>* nn = mytree.nn(&pointset[j]);
      annresult.push_back(pair<Point<T>, Point<T> >(pointset[j], *nn));
   }
   anntime = annutil::microsec(anntime);
   
   if (argc > 5 && atoi(argv[5]) == 1) {
      double chktime = annutil::microsec();
      ANNComparator<T, Point<T> > comp(false);
      
      int errors = comp.doComparison(annresult, pointset, atoi(argv[4]));
      chktime = annutil::microsec(chktime);
      
      if (atoi(argv[4]) > 0) {
         annutil::printResultWithCheck(progname, pointset.size(), annresult.size(), atoi(argv[3]), 
                                       errors, buildtime, anntime, chktime);
      }
      exit(errors);
   }
   else { 
      annutil::printResultWithoutCheck(progname, pointset.size(), annresult.size(), atoi(argv[3]),
                                       buildtime, anntime);
      exit(0);
   }


   return 0;
}

