/* Classes that are used as functors, i.e. I need to
+  pass different comparators around, depending on whether
+  I do ANN to the left or to the right.
+
+  Originally made for the All Nearest Neighbor plane sweep algorithm
+  (Hinrichs et al.)
+  Michael Neidhardt June 2008.
*/

#ifndef COMPARE_H
#define COMPARE_H

template< typename T >
class intervalChecker {
   public:
   // Horizontal interval: When doing ANN to the left, x must be greater than a
   // or less than/equal to b to be inside interval.
   // a and b are x-values, and a < b.
   bool  inIntervalANNleft(T x, T a, T b) { return (x > a && x <= b); }
   // Horizontal interval: When doing ANN to the right, x must be less than a
   // or greater than/equal to b to be inside interval.
   // a and b are x-values, and a < b.
   bool inIntervalANNright(T x, T a, T b) { return (x < a && x >= b); }



   // Vertical interval: Check whether current is in the interval between
   // p.y() and the intersection between nextBS and sweep line.
   bool   notInIntervalUP(Bisector<T>& current, Bisector<T>& next, Point<T>& p) {
      if (current.get_b() == 0) { 
         return true;
      }
      else if (next.get_b() == 0) {
         if (current.getYValueAsDouble(p.x()) < p.y()) { return true; }
         else                                      { return false; }
      }
      else { // if (next.get_b() != 0) {
         if (current.getYValueAsDouble(p.x()) < p.y() || 
             current.getYValueAsDouble(p.x()) > next.getYValueAsDouble(p.x())) {
            return true;
         }
         else {
            return false;     // I.e. current is in the interval.
         }
      }
   }

   // Vertical interval: Check whether current is in the interval between
   // p.y() and the intersection between nextBS and sweep line.
   bool   notInIntervalDOWN(Bisector<T>& current, Bisector<T>& next, Point<T>& p) {
      if (current.get_b() == 0) { 
         return true;
      }
      else if (next.get_b() == 0) {
         if (current.getYValueAsDouble(p.x()) > p.y()) { return true; }
         else                                      { return false; }
      }
      else { // if (next.get_b() != 0) {
         if (current.getYValueAsDouble(p.x()) > p.y() || 
             current.getYValueAsDouble(p.x()) < next.getYValueAsDouble(p.x())) {
            return true;
         }
         else {
            return false;     // I.e. current is in the interval.
         }
      }
   }

};



#endif
