/* Test program for the full ANNPlanesweep program. This version uses ANNComparison
+  to check if it is correct.
*/
#include <iostream>
#include <vector>
#include "annutil.h"
#include "ANNPlaneSweep.h"
#include "ANNComparator2.h"
#include "Point.h"
#include "Datasource2.h"

using namespace std;

typedef double T;

int main(int argc, char** argv) {
   std::string progname = "FPSweep";

   if (argc < 2) {
      cerr << progname
           << "\nSyntax: " << argv[0] << " pointtype [filename|numOfPoints] "
           <<                            " [precision] [verbose] [check]\n\n"
           << "   pointtype=0 means the following arg is filename with points.\n" 
           << "   precision: must be in [1,31]. Decides precision of random numbers.\n"
           << "   verbose: 0=quiet, 1=one-line summary, 2=all points listed, 3=also debug info.\n"
           << "   check:   1 means check against brute force results, 0 means no check.\n\n";

      exit(0);
   }

   Datasource2<T,Point<T> > dsrc;
   vector<Point<T> > pointset;
   
   if (atoi(argv[1]) == 0) {
      pointset = dsrc.readPointset(argv[2]);
   }
   else {   // Al inddata kommer nu fra en fil.
      std::cerr << "Input must come from a file. First arg. should be 0.\n";
      exit(1);
      //pointset = dsrc.getRandomIntPoints(atoi(argv[2]), atoi(argv[3]));
   }

   ANNPlaneSweep<T> sweeper(pointset);
   double sweeptime = annutil::microsec();
   vector<pair<Point<T>, Point<T> > > sweepresult = sweeper.findAllNearestNeighbors();
   sweeptime = annutil::microsec(sweeptime);

   if (argc > 5 && atoi(argv[5]) == 1) {
      double chktime = annutil::microsec();
      ANNComparator<T, Point<T> > comp;

      int errors = comp.doComparison(sweepresult, pointset, atoi(argv[4]));
      chktime = annutil::microsec(chktime);

      if (atoi(argv[4]) > 0) {
         annutil::printResultWithCheck(progname, pointset.size(), sweepresult.size(), 
                             atoi(argv[3]), errors, 0, sweeptime, chktime);
      }
      exit(errors);
   }
   else { 
      annutil::printResultWithoutCheck(progname, pointset.size(), sweepresult.size(),
                                       atoi(argv[3]), 0, sweeptime);
      exit(0);
   }
}

