/* Classes that are used as functors, i.e. I need to
+  pass different comparators around, depending on whether
+  I do ANN to the left or to the right.
+
+  Originally made for the All Nearest Neighbor plane sweep algorithm
+  (Hinrichs et al.)
+  Michael Neidhardt June 2008.
*/

#ifndef INTERVALFUNCTIONS_RAT_H
#define INTERVALFUNCTIONS_RAT_H

template< typename T >
class intervalChecker {
   public:
   // Vertical interval: Check whether current is in the interval between
   // p.y() and the intersection between nextBS and sweep line.
   bool   notInIntervalUP(Bisector<T>& current, Bisector<T>& next, Point<T>& p) {
      if (current.get_b() == 0) { 
         return true;
      }
      else if (next.get_b() == 0) {
         if (current.getYValue(p.x()) < p.y()) { return true; }
         else                                  { return false; }
      }
      else { // if (next.get_b() != 0) {
         if (current.getYValue(p.x()) < p.y() || 
             current.getYValue(p.x()) > next.getYValue(p.x())) {
            return true;
         }
         else {
            return false;     // I.e. current is in the interval.
         }
      }
   }

   // Vertical interval: Check whether current is in the interval between
   // p.y() and the intersection between nextBS and sweep line.
   bool   notInIntervalDOWN(Bisector<T>& current, Bisector<T>& next, Point<T>& p) {
      if (current.get_b() == 0) { 
         return true;
      }
      else if (next.get_b() == 0) {
         if (current.getYValue(p.x()) > p.y()) { return true; }
         else                                      { return false; }
      }
      else { // if (next.get_b() != 0) {
         if (current.getYValue(p.x()) > p.y() || 
             current.getYValue(p.x()) < next.getYValue(p.x())) {
            return true;
         }
         else {
            return false;     // I.e. current is in the interval.
         }
      }
   }

};



#endif
