/* Test program for the full ANNPlanesweep program. */


#include <iostream>
#include <sstream>
#include <vector>
#include <string>
#include <LEDA/numbers/rational.h>
#include <LEDA/numbers/integer.h>
//#include <boost/rational.hpp>
#include "annutil.h"
#include "ANNPlaneSweep_rat.h"
#include "ANNComparator2.h"
#include "Point.h"
#include "Datasource2.h"

using namespace std;
typedef leda::rational T;
//typedef boost::rational<int> T;


int main(int argc, char** argv) {

   std::string progname = "RATSweep";
   if (argc < 2) {
      cerr << "This runs ANN Plane sweep with LEDA rational as type for coordinates\n"
           << "Syntax: " << argv[0] << " pointtype [filename|numOfPoints] "
           <<                          " [precision] [verbose] [check]\n\n"
           << "   pointtype=0 means the following arg is filename with points.\n" 
           << "   precision: must be in [1,31]. Decides precision of random numbers.\n"
           << "   verbose: 0=quiet, 1=one-line summary, 2=all points listed, 3=also debug info.\n"
           << "   check:   1 means check against brute force results, 0 means no check.\n\n";

      exit(0);
   }

   Datasource2<T, Point<T> > dsrc;
   vector<Point<T> > pointset;
   
   if (atoi(argv[1]) == 0) {
      pointset = dsrc.readLedaRationalPointset(argv[2]);
   }
   else {      // Now the data must be made and stored in a file. Then this program reads the file.
      std::cerr << "Input must come from a file. First arg. should be 0.\n";
      exit(1);
   }

   ANNPlaneSweep<T> sweeper(pointset);
   double sweeptime = annutil::microsec();
   vector<pair<Point<T>, Point<T> > > sweepresult = sweeper.findAllNearestNeighbors();
   sweeptime = annutil::microsec(sweeptime);


   if (argc > 5 && atoi(argv[5]) == 1) {
      double chktime = annutil::microsec();
      ANNComparator<T,Point<T> > comp;
      int errors = comp.doComparison(sweepresult, pointset, atoi(argv[4]));
      chktime = annutil::microsec(chktime);

      if (atoi(argv[4]) > 0) {
         annutil::printResultWithCheck(progname, pointset.size(), sweepresult.size(), 
                             atoi(argv[3]), errors, 0, sweeptime, chktime);
      }
      exit(errors);
   }
   else { 
      annutil::printResultWithoutCheck(progname, pointset.size(), sweepresult.size(),
                                       atoi(argv[3]), 0, sweeptime);
      exit(0);
   }

// The reason for outputting size of sweepresult is that it may have shrunk.
// This happens if pointset is generated by getRandomPoints_rat, 
// precision is low and pointset.size is large, in which case you get a 
// lot of random points from a small space, say 2^3 or 2^4.

   return 0;
}

